# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "monitor metrics alert list",
)
class List(AAZCommand):
    """List metric-based alert rules.

    :example: List metric-based alert rules.
        az monitor metrics alert list --resource-group MyResourceGroup
    """

    _aaz_info = {
        "version": "2018-03-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/providers/microsoft.insights/metricalerts", "2018-03-01"],
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.insights/metricalerts", "2018-03-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.resource_group = AAZResourceGroupNameArg()
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        condition_0 = has_value(self.ctx.args.resource_group) and has_value(self.ctx.subscription_id)
        condition_1 = has_value(self.ctx.subscription_id) and has_value(self.ctx.args.resource_group) is not True
        if condition_0:
            self.MetricAlertsListByResourceGroup(ctx=self.ctx)()
        if condition_1:
            self.MetricAlertsListBySubscription(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance.value, client_flatten=True)
        return result

    class MetricAlertsListByResourceGroup(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/metricAlerts",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2018-03-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.value = AAZListType()

            value = cls._schema_on_200.value
            value.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element
            _element.id = AAZStrType(
                flags={"read_only": True},
            )
            _element.location = AAZStrType(
                flags={"required": True},
            )
            _element.name = AAZStrType(
                flags={"read_only": True},
            )
            _element.properties = AAZObjectType(
                flags={"required": True, "client_flatten": True},
            )
            _element.tags = AAZDictType()
            _element.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.value.Element.properties
            properties.actions = AAZListType()
            properties.auto_mitigate = AAZBoolType(
                serialized_name="autoMitigate",
            )
            properties.criteria = AAZObjectType(
                flags={"required": True},
            )
            properties.description = AAZStrType()
            properties.enabled = AAZBoolType(
                flags={"required": True},
            )
            properties.evaluation_frequency = AAZStrType(
                serialized_name="evaluationFrequency",
                flags={"required": True},
            )
            properties.is_migrated = AAZBoolType(
                serialized_name="isMigrated",
                flags={"read_only": True},
            )
            properties.last_updated_time = AAZStrType(
                serialized_name="lastUpdatedTime",
                flags={"read_only": True},
            )
            properties.scopes = AAZListType(
                flags={"required": True},
            )
            properties.severity = AAZIntType(
                flags={"required": True},
            )
            properties.target_resource_region = AAZStrType(
                serialized_name="targetResourceRegion",
            )
            properties.target_resource_type = AAZStrType(
                serialized_name="targetResourceType",
            )
            properties.window_size = AAZStrType(
                serialized_name="windowSize",
                flags={"required": True},
            )

            actions = cls._schema_on_200.value.Element.properties.actions
            actions.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.actions.Element
            _element.action_group_id = AAZStrType(
                serialized_name="actionGroupId",
            )
            _element.web_hook_properties = AAZDictType(
                serialized_name="webHookProperties",
            )

            web_hook_properties = cls._schema_on_200.value.Element.properties.actions.Element.web_hook_properties
            web_hook_properties.Element = AAZStrType()

            criteria = cls._schema_on_200.value.Element.properties.criteria
            criteria["odata.type"] = AAZStrType(
                flags={"required": True},
            )

            disc_microsoft__azure__monitor__multiple_resource_multiple_metric_criteria = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria")
            disc_microsoft__azure__monitor__multiple_resource_multiple_metric_criteria.all_of = AAZListType(
                serialized_name="allOf",
            )

            all_of = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of
            all_of.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of.Element
            _element.criterion_type = AAZStrType(
                serialized_name="criterionType",
                flags={"required": True},
            )
            _element.dimensions = AAZListType()
            _element.metric_name = AAZStrType(
                serialized_name="metricName",
                flags={"required": True},
            )
            _element.metric_namespace = AAZStrType(
                serialized_name="metricNamespace",
            )
            _element.name = AAZStrType(
                flags={"required": True},
            )
            _element.skip_metric_validation = AAZBoolType(
                serialized_name="skipMetricValidation",
            )
            _element.time_aggregation = AAZStrType(
                serialized_name="timeAggregation",
                flags={"required": True},
            )

            dimensions = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of.Element.dimensions
            dimensions.Element = AAZObjectType()
            _ListHelper._build_schema_metric_dimension_read(dimensions.Element)

            disc_dynamic_threshold_criterion = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of.Element.discriminate_by("criterion_type", "DynamicThresholdCriterion")
            disc_dynamic_threshold_criterion.alert_sensitivity = AAZStrType(
                serialized_name="alertSensitivity",
                flags={"required": True},
            )
            disc_dynamic_threshold_criterion.failing_periods = AAZObjectType(
                serialized_name="failingPeriods",
                flags={"required": True},
            )
            disc_dynamic_threshold_criterion.ignore_data_before = AAZStrType(
                serialized_name="ignoreDataBefore",
            )
            disc_dynamic_threshold_criterion.operator = AAZStrType(
                flags={"required": True},
            )

            failing_periods = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of.Element.discriminate_by("criterion_type", "DynamicThresholdCriterion").failing_periods
            failing_periods.min_failing_periods_to_alert = AAZFloatType(
                serialized_name="minFailingPeriodsToAlert",
                flags={"required": True},
            )
            failing_periods.number_of_evaluation_periods = AAZFloatType(
                serialized_name="numberOfEvaluationPeriods",
                flags={"required": True},
            )

            disc_static_threshold_criterion = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of.Element.discriminate_by("criterion_type", "StaticThresholdCriterion")
            disc_static_threshold_criterion.operator = AAZStrType(
                flags={"required": True},
            )
            disc_static_threshold_criterion.threshold = AAZFloatType(
                flags={"required": True},
            )

            disc_microsoft__azure__monitor__single_resource_multiple_metric_criteria = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.SingleResourceMultipleMetricCriteria")
            disc_microsoft__azure__monitor__single_resource_multiple_metric_criteria.all_of = AAZListType(
                serialized_name="allOf",
            )

            all_of = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.SingleResourceMultipleMetricCriteria").all_of
            all_of.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.SingleResourceMultipleMetricCriteria").all_of.Element
            _element.criterion_type = AAZStrType(
                serialized_name="criterionType",
                flags={"required": True},
            )
            _element.dimensions = AAZListType()
            _element.metric_name = AAZStrType(
                serialized_name="metricName",
                flags={"required": True},
            )
            _element.metric_namespace = AAZStrType(
                serialized_name="metricNamespace",
            )
            _element.name = AAZStrType(
                flags={"required": True},
            )
            _element.operator = AAZStrType(
                flags={"required": True},
            )
            _element.skip_metric_validation = AAZBoolType(
                serialized_name="skipMetricValidation",
            )
            _element.threshold = AAZFloatType(
                flags={"required": True},
            )
            _element.time_aggregation = AAZStrType(
                serialized_name="timeAggregation",
                flags={"required": True},
            )

            dimensions = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.SingleResourceMultipleMetricCriteria").all_of.Element.dimensions
            dimensions.Element = AAZObjectType()
            _ListHelper._build_schema_metric_dimension_read(dimensions.Element)

            disc_microsoft__azure__monitor__webtest_location_availability_criteria = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.WebtestLocationAvailabilityCriteria")
            disc_microsoft__azure__monitor__webtest_location_availability_criteria.component_id = AAZStrType(
                serialized_name="componentId",
                flags={"required": True},
            )
            disc_microsoft__azure__monitor__webtest_location_availability_criteria.failed_location_count = AAZFloatType(
                serialized_name="failedLocationCount",
                flags={"required": True},
            )
            disc_microsoft__azure__monitor__webtest_location_availability_criteria.web_test_id = AAZStrType(
                serialized_name="webTestId",
                flags={"required": True},
            )

            scopes = cls._schema_on_200.value.Element.properties.scopes
            scopes.Element = AAZStrType()

            tags = cls._schema_on_200.value.Element.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200

    class MetricAlertsListBySubscription(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/providers/Microsoft.Insights/metricAlerts",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2018-03-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.value = AAZListType()

            value = cls._schema_on_200.value
            value.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element
            _element.id = AAZStrType(
                flags={"read_only": True},
            )
            _element.location = AAZStrType(
                flags={"required": True},
            )
            _element.name = AAZStrType(
                flags={"read_only": True},
            )
            _element.properties = AAZObjectType(
                flags={"required": True, "client_flatten": True},
            )
            _element.tags = AAZDictType()
            _element.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.value.Element.properties
            properties.actions = AAZListType()
            properties.auto_mitigate = AAZBoolType(
                serialized_name="autoMitigate",
            )
            properties.criteria = AAZObjectType(
                flags={"required": True},
            )
            properties.description = AAZStrType()
            properties.enabled = AAZBoolType(
                flags={"required": True},
            )
            properties.evaluation_frequency = AAZStrType(
                serialized_name="evaluationFrequency",
                flags={"required": True},
            )
            properties.is_migrated = AAZBoolType(
                serialized_name="isMigrated",
                flags={"read_only": True},
            )
            properties.last_updated_time = AAZStrType(
                serialized_name="lastUpdatedTime",
                flags={"read_only": True},
            )
            properties.scopes = AAZListType(
                flags={"required": True},
            )
            properties.severity = AAZIntType(
                flags={"required": True},
            )
            properties.target_resource_region = AAZStrType(
                serialized_name="targetResourceRegion",
            )
            properties.target_resource_type = AAZStrType(
                serialized_name="targetResourceType",
            )
            properties.window_size = AAZStrType(
                serialized_name="windowSize",
                flags={"required": True},
            )

            actions = cls._schema_on_200.value.Element.properties.actions
            actions.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.actions.Element
            _element.action_group_id = AAZStrType(
                serialized_name="actionGroupId",
            )
            _element.web_hook_properties = AAZDictType(
                serialized_name="webHookProperties",
            )

            web_hook_properties = cls._schema_on_200.value.Element.properties.actions.Element.web_hook_properties
            web_hook_properties.Element = AAZStrType()

            criteria = cls._schema_on_200.value.Element.properties.criteria
            criteria["odata.type"] = AAZStrType(
                flags={"required": True},
            )

            disc_microsoft__azure__monitor__multiple_resource_multiple_metric_criteria = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria")
            disc_microsoft__azure__monitor__multiple_resource_multiple_metric_criteria.all_of = AAZListType(
                serialized_name="allOf",
            )

            all_of = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of
            all_of.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of.Element
            _element.criterion_type = AAZStrType(
                serialized_name="criterionType",
                flags={"required": True},
            )
            _element.dimensions = AAZListType()
            _element.metric_name = AAZStrType(
                serialized_name="metricName",
                flags={"required": True},
            )
            _element.metric_namespace = AAZStrType(
                serialized_name="metricNamespace",
            )
            _element.name = AAZStrType(
                flags={"required": True},
            )
            _element.skip_metric_validation = AAZBoolType(
                serialized_name="skipMetricValidation",
            )
            _element.time_aggregation = AAZStrType(
                serialized_name="timeAggregation",
                flags={"required": True},
            )

            dimensions = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of.Element.dimensions
            dimensions.Element = AAZObjectType()
            _ListHelper._build_schema_metric_dimension_read(dimensions.Element)

            disc_dynamic_threshold_criterion = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of.Element.discriminate_by("criterion_type", "DynamicThresholdCriterion")
            disc_dynamic_threshold_criterion.alert_sensitivity = AAZStrType(
                serialized_name="alertSensitivity",
                flags={"required": True},
            )
            disc_dynamic_threshold_criterion.failing_periods = AAZObjectType(
                serialized_name="failingPeriods",
                flags={"required": True},
            )
            disc_dynamic_threshold_criterion.ignore_data_before = AAZStrType(
                serialized_name="ignoreDataBefore",
            )
            disc_dynamic_threshold_criterion.operator = AAZStrType(
                flags={"required": True},
            )

            failing_periods = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of.Element.discriminate_by("criterion_type", "DynamicThresholdCriterion").failing_periods
            failing_periods.min_failing_periods_to_alert = AAZFloatType(
                serialized_name="minFailingPeriodsToAlert",
                flags={"required": True},
            )
            failing_periods.number_of_evaluation_periods = AAZFloatType(
                serialized_name="numberOfEvaluationPeriods",
                flags={"required": True},
            )

            disc_static_threshold_criterion = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of.Element.discriminate_by("criterion_type", "StaticThresholdCriterion")
            disc_static_threshold_criterion.operator = AAZStrType(
                flags={"required": True},
            )
            disc_static_threshold_criterion.threshold = AAZFloatType(
                flags={"required": True},
            )

            disc_microsoft__azure__monitor__single_resource_multiple_metric_criteria = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.SingleResourceMultipleMetricCriteria")
            disc_microsoft__azure__monitor__single_resource_multiple_metric_criteria.all_of = AAZListType(
                serialized_name="allOf",
            )

            all_of = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.SingleResourceMultipleMetricCriteria").all_of
            all_of.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.SingleResourceMultipleMetricCriteria").all_of.Element
            _element.criterion_type = AAZStrType(
                serialized_name="criterionType",
                flags={"required": True},
            )
            _element.dimensions = AAZListType()
            _element.metric_name = AAZStrType(
                serialized_name="metricName",
                flags={"required": True},
            )
            _element.metric_namespace = AAZStrType(
                serialized_name="metricNamespace",
            )
            _element.name = AAZStrType(
                flags={"required": True},
            )
            _element.operator = AAZStrType(
                flags={"required": True},
            )
            _element.skip_metric_validation = AAZBoolType(
                serialized_name="skipMetricValidation",
            )
            _element.threshold = AAZFloatType(
                flags={"required": True},
            )
            _element.time_aggregation = AAZStrType(
                serialized_name="timeAggregation",
                flags={"required": True},
            )

            dimensions = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.SingleResourceMultipleMetricCriteria").all_of.Element.dimensions
            dimensions.Element = AAZObjectType()
            _ListHelper._build_schema_metric_dimension_read(dimensions.Element)

            disc_microsoft__azure__monitor__webtest_location_availability_criteria = cls._schema_on_200.value.Element.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.WebtestLocationAvailabilityCriteria")
            disc_microsoft__azure__monitor__webtest_location_availability_criteria.component_id = AAZStrType(
                serialized_name="componentId",
                flags={"required": True},
            )
            disc_microsoft__azure__monitor__webtest_location_availability_criteria.failed_location_count = AAZFloatType(
                serialized_name="failedLocationCount",
                flags={"required": True},
            )
            disc_microsoft__azure__monitor__webtest_location_availability_criteria.web_test_id = AAZStrType(
                serialized_name="webTestId",
                flags={"required": True},
            )

            scopes = cls._schema_on_200.value.Element.properties.scopes
            scopes.Element = AAZStrType()

            tags = cls._schema_on_200.value.Element.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200


class _ListHelper:
    """Helper class for List"""

    _schema_metric_dimension_read = None

    @classmethod
    def _build_schema_metric_dimension_read(cls, _schema):
        if cls._schema_metric_dimension_read is not None:
            _schema.name = cls._schema_metric_dimension_read.name
            _schema.operator = cls._schema_metric_dimension_read.operator
            _schema.values = cls._schema_metric_dimension_read.values
            return

        cls._schema_metric_dimension_read = _schema_metric_dimension_read = AAZObjectType()

        metric_dimension_read = _schema_metric_dimension_read
        metric_dimension_read.name = AAZStrType(
            flags={"required": True},
        )
        metric_dimension_read.operator = AAZStrType(
            flags={"required": True},
        )
        metric_dimension_read.values = AAZListType(
            flags={"required": True},
        )

        values = _schema_metric_dimension_read.values
        values.Element = AAZStrType()

        _schema.name = cls._schema_metric_dimension_read.name
        _schema.operator = cls._schema_metric_dimension_read.operator
        _schema.values = cls._schema_metric_dimension_read.values


__all__ = ["List"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "consumption budget create-with-rg",
    is_preview=True,
)
class CreateWithRg(AAZCommand):
    """Create operation to create or update a budget. Update operation requires latest eTag to be set in the request mandatorily. You may obtain the latest eTag by performing a get operation. Create operation does not require eTag.
    """

    _aaz_info = {
        "version": "2023-05-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.consumption/budgets/{}", "2023-05-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.budget_name = AAZStrArg(
            options=["-n", "--name", "--budget-name"],
            help="Budget Name.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )

        # define Arg Group "Parameters"

        _args_schema = cls._args_schema
        _args_schema.e_tag = AAZStrArg(
            options=["--e-tag"],
            arg_group="Parameters",
            help="eTag of the resource. To handle concurrent update scenario, this field will be used to determine whether the user is updating the latest version or not.",
        )

        # define Arg Group "Properties"

        _args_schema = cls._args_schema
        _args_schema.amount = AAZFloatArg(
            options=["--amount"],
            arg_group="Properties",
            help="The total amount of cost to track with the budget",
        )
        _args_schema.category = AAZStrArg(
            options=["--category"],
            arg_group="Properties",
            help="The category of the budget, whether the budget tracks cost or usage.",
            enum={"Cost": "Cost", "Usage": "Usage"},
        )
        _args_schema.filters = AAZObjectArg(
            options=["--filters"],
            arg_group="Properties",
            help="May be used to filter budgets by resource group, resource, or meter.",
        )
        _args_schema.notifications = AAZDictArg(
            options=["--notifications"],
            arg_group="Properties",
            help="Dictionary of notifications associated with the budget. Budget can have up to five notifications.",
        )
        _args_schema.time_grain = AAZStrArg(
            options=["--time-grain"],
            arg_group="Properties",
            help="The time covered by a budget. Tracking of the amount will be reset based on the time grain.",
            enum={"Annually": "Annually", "Monthly": "Monthly", "Quarterly": "Quarterly"},
        )
        _args_schema.time_period = AAZObjectArg(
            options=["--time-period"],
            arg_group="Properties",
            help="Has start and end date of the budget. The start date must be first of the month and should be less than the end date. Budget start date must be on or after June 1, 2017. Future start date should not be more than three months. Past start date should  be selected within the timegrain period. There are no restrictions on the end date.",
        )

        filters = cls._args_schema.filters
        filters.meters = AAZListArg(
            options=["meters"],
            help="The list of filters on meters, mandatory for budgets of usage category. ",
        )
        filters.resource_groups = AAZListArg(
            options=["resource-groups"],
            help="The list of filters on resource groups, allowed at subscription level only.",
        )
        filters.resources = AAZListArg(
            options=["resources"],
            help="The list of filters on resources.",
        )

        meters = cls._args_schema.filters.meters
        meters.Element = AAZStrArg()

        resource_groups = cls._args_schema.filters.resource_groups
        resource_groups.Element = AAZStrArg()

        resources = cls._args_schema.filters.resources
        resources.Element = AAZStrArg()

        notifications = cls._args_schema.notifications
        notifications.Element = AAZObjectArg()

        _element = cls._args_schema.notifications.Element
        _element.contact_emails = AAZListArg(
            options=["contact-emails"],
            help="Email addresses to send the budget notification to when the threshold is exceeded.",
            required=True,
        )
        _element.contact_groups = AAZListArg(
            options=["contact-groups"],
            help="Action groups to send the budget notification to when the threshold is exceeded.",
        )
        _element.contact_roles = AAZListArg(
            options=["contact-roles"],
            help="Contact roles to send the budget notification to when the threshold is exceeded.",
        )
        _element.enabled = AAZBoolArg(
            options=["enabled"],
            help="The notification is enabled or not.",
            required=True,
        )
        _element.operator = AAZStrArg(
            options=["operator"],
            help="The comparison operator.",
            required=True,
            enum={"EqualTo": "EqualTo", "GreaterThan": "GreaterThan", "GreaterThanOrEqualTo": "GreaterThanOrEqualTo"},
        )
        _element.threshold = AAZFloatArg(
            options=["threshold"],
            help="Threshold value associated with a notification. Notification is sent when the cost exceeded the threshold. It is always percent and has to be between 0 and 1000.",
            required=True,
        )

        contact_emails = cls._args_schema.notifications.Element.contact_emails
        contact_emails.Element = AAZStrArg()

        contact_groups = cls._args_schema.notifications.Element.contact_groups
        contact_groups.Element = AAZStrArg()

        contact_roles = cls._args_schema.notifications.Element.contact_roles
        contact_roles.Element = AAZStrArg()

        time_period = cls._args_schema.time_period
        time_period.end_date = AAZDateTimeArg(
            options=["end-date"],
            help="The end date for the budget. If not provided, we default this to 10 years from the start date.",
        )
        time_period.start_date = AAZDateTimeArg(
            options=["start-date"],
            help="The start date for the budget.",
            required=True,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.BudgetsCreateOrUpdateByResourceGroupName(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class BudgetsCreateOrUpdateByResourceGroupName(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200, 201]:
                return self.on_200_201(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Consumption/budgets/{budgetName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "budgetName", self.ctx.args.budget_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2023-05-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("eTag", AAZStrType, ".e_tag")
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("amount", AAZFloatType, ".amount", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("category", AAZStrType, ".category", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("filters", AAZObjectType, ".filters")
                properties.set_prop("notifications", AAZDictType, ".notifications")
                properties.set_prop("timeGrain", AAZStrType, ".time_grain", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("timePeriod", AAZObjectType, ".time_period", typ_kwargs={"flags": {"required": True}})

            filters = _builder.get(".properties.filters")
            if filters is not None:
                filters.set_prop("meters", AAZListType, ".meters")
                filters.set_prop("resourceGroups", AAZListType, ".resource_groups")
                filters.set_prop("resources", AAZListType, ".resources")

            meters = _builder.get(".properties.filters.meters")
            if meters is not None:
                meters.set_elements(AAZStrType, ".")

            resource_groups = _builder.get(".properties.filters.resourceGroups")
            if resource_groups is not None:
                resource_groups.set_elements(AAZStrType, ".")

            resources = _builder.get(".properties.filters.resources")
            if resources is not None:
                resources.set_elements(AAZStrType, ".")

            notifications = _builder.get(".properties.notifications")
            if notifications is not None:
                notifications.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.notifications{}")
            if _elements is not None:
                _elements.set_prop("contactEmails", AAZListType, ".contact_emails", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("contactGroups", AAZListType, ".contact_groups")
                _elements.set_prop("contactRoles", AAZListType, ".contact_roles")
                _elements.set_prop("enabled", AAZBoolType, ".enabled", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("operator", AAZStrType, ".operator", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("threshold", AAZFloatType, ".threshold", typ_kwargs={"flags": {"required": True}})

            contact_emails = _builder.get(".properties.notifications{}.contactEmails")
            if contact_emails is not None:
                contact_emails.set_elements(AAZStrType, ".")

            contact_groups = _builder.get(".properties.notifications{}.contactGroups")
            if contact_groups is not None:
                contact_groups.set_elements(AAZStrType, ".")

            contact_roles = _builder.get(".properties.notifications{}.contactRoles")
            if contact_roles is not None:
                contact_roles.set_elements(AAZStrType, ".")

            time_period = _builder.get(".properties.timePeriod")
            if time_period is not None:
                time_period.set_prop("endDate", AAZStrType, ".end_date")
                time_period.set_prop("startDate", AAZStrType, ".start_date", typ_kwargs={"flags": {"required": True}})

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()

            _schema_on_200_201 = cls._schema_on_200_201
            _schema_on_200_201.e_tag = AAZStrType(
                serialized_name="eTag",
            )
            _schema_on_200_201.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200_201.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200_201.properties
            properties.amount = AAZFloatType(
                flags={"required": True},
            )
            properties.category = AAZStrType(
                flags={"required": True},
            )
            properties.current_spend = AAZObjectType(
                serialized_name="currentSpend",
            )
            properties.filters = AAZObjectType()
            properties.notifications = AAZDictType()
            properties.time_grain = AAZStrType(
                serialized_name="timeGrain",
                flags={"required": True},
            )
            properties.time_period = AAZObjectType(
                serialized_name="timePeriod",
                flags={"required": True},
            )

            current_spend = cls._schema_on_200_201.properties.current_spend
            current_spend.amount = AAZFloatType(
                flags={"read_only": True},
            )
            current_spend.unit = AAZStrType(
                flags={"read_only": True},
            )

            filters = cls._schema_on_200_201.properties.filters
            filters.meters = AAZListType()
            filters.resource_groups = AAZListType(
                serialized_name="resourceGroups",
            )
            filters.resources = AAZListType()

            meters = cls._schema_on_200_201.properties.filters.meters
            meters.Element = AAZStrType()

            resource_groups = cls._schema_on_200_201.properties.filters.resource_groups
            resource_groups.Element = AAZStrType()

            resources = cls._schema_on_200_201.properties.filters.resources
            resources.Element = AAZStrType()

            notifications = cls._schema_on_200_201.properties.notifications
            notifications.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.notifications.Element
            _element.contact_emails = AAZListType(
                serialized_name="contactEmails",
                flags={"required": True},
            )
            _element.contact_groups = AAZListType(
                serialized_name="contactGroups",
            )
            _element.contact_roles = AAZListType(
                serialized_name="contactRoles",
            )
            _element.enabled = AAZBoolType(
                flags={"required": True},
            )
            _element.operator = AAZStrType(
                flags={"required": True},
            )
            _element.threshold = AAZFloatType(
                flags={"required": True},
            )

            contact_emails = cls._schema_on_200_201.properties.notifications.Element.contact_emails
            contact_emails.Element = AAZStrType()

            contact_groups = cls._schema_on_200_201.properties.notifications.Element.contact_groups
            contact_groups.Element = AAZStrType()

            contact_roles = cls._schema_on_200_201.properties.notifications.Element.contact_roles
            contact_roles.Element = AAZStrType()

            time_period = cls._schema_on_200_201.properties.time_period
            time_period.end_date = AAZStrType(
                serialized_name="endDate",
            )
            time_period.start_date = AAZStrType(
                serialized_name="startDate",
                flags={"required": True},
            )

            return cls._schema_on_200_201


class _CreateWithRgHelper:
    """Helper class for CreateWithRg"""


__all__ = ["CreateWithRg"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "network express-route gateway create",
)
class Create(AAZCommand):
    """Create an ExpressRoute gateway.

    :example: Create an ExpressRoute gateway.
        az network express-route gateway create --name MyExpressRouteGateway --resource-group MyResourceGroup --min-val 5
    """

    _aaz_info = {
        "version": "2023-09-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/expressroutegateways/{}", "2023-09-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.name = AAZStrArg(
            options=["-n", "--name"],
            help="ExpressRoute gateway name.",
            required=True,
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.location = AAZResourceLocationArg(
            help="Location. Values from: `az account list-locations`. You can configure the default location using `az configure --defaults location=<location>`.",
            fmt=AAZResourceLocationArgFormat(
                resource_group_arg="resource_group",
            ),
        )
        _args_schema.allow_non_vwan_traffic = AAZBoolArg(
            options=["--allow-non-vwan-traffic"],
            help="Configures this gateway to accept traffic from non Virtual WAN networks.",
        )
        _args_schema.virtual_hub = AAZStrArg(
            options=["--virtual-hub"],
            help="Name or ID of the virtual hub to associate with the gateway.",
        )
        _args_schema.tags = AAZDictArg(
            options=["--tags"],
            help="Space-separated tags: key[=value] [key[=value] ...]. Use \"\" to clear existing tags.",
        )

        tags = cls._args_schema.tags
        tags.Element = AAZStrArg()

        # define Arg Group "Autoscale"

        _args_schema = cls._args_schema
        _args_schema.max_val = AAZIntArg(
            options=["--max-val"],
            arg_group="Autoscale",
            help="Maximum number of scale units deployed for gateway.",
        )
        _args_schema.min_val = AAZIntArg(
            options=["--min-val"],
            arg_group="Autoscale",
            help="Minimum number of scale units deployed for gateway.  Default: 2.",
            default=2,
        )

        # define Arg Group "Properties"

        # define Arg Group "PutExpressRouteGatewayParameters"
        return cls._args_schema

    _args_sub_resource_create = None

    @classmethod
    def _build_args_sub_resource_create(cls, _schema):
        if cls._args_sub_resource_create is not None:
            _schema.id = cls._args_sub_resource_create.id
            return

        cls._args_sub_resource_create = AAZObjectArg()

        sub_resource_create = cls._args_sub_resource_create
        sub_resource_create.id = AAZStrArg(
            options=["id"],
            help="Resource ID.",
        )

        _schema.id = cls._args_sub_resource_create.id

    def _execute_operations(self):
        self.pre_operations()
        yield self.ExpressRouteGatewaysCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class ExpressRouteGatewaysCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200, 201]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/expressRouteGateways/{expressRouteGatewayName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "expressRouteGatewayName", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2023-09-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("location", AAZStrType, ".location")
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})
            _builder.set_prop("tags", AAZDictType, ".tags")

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("allowNonVirtualWanTraffic", AAZBoolType, ".allow_non_vwan_traffic")
                properties.set_prop("autoScaleConfiguration", AAZObjectType)
                properties.set_prop("virtualHub", AAZObjectType, ".", typ_kwargs={"flags": {"required": True}})

            auto_scale_configuration = _builder.get(".properties.autoScaleConfiguration")
            if auto_scale_configuration is not None:
                auto_scale_configuration.set_prop("bounds", AAZObjectType)

            bounds = _builder.get(".properties.autoScaleConfiguration.bounds")
            if bounds is not None:
                bounds.set_prop("max", AAZIntType, ".max_val")
                bounds.set_prop("min", AAZIntType, ".min_val")

            virtual_hub = _builder.get(".properties.virtualHub")
            if virtual_hub is not None:
                virtual_hub.set_prop("id", AAZStrType, ".virtual_hub")

            tags = _builder.get(".tags")
            if tags is not None:
                tags.set_elements(AAZStrType, ".")

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()

            _schema_on_200_201 = cls._schema_on_200_201
            _schema_on_200_201.etag = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.id = AAZStrType()
            _schema_on_200_201.location = AAZStrType()
            _schema_on_200_201.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200_201.tags = AAZDictType()
            _schema_on_200_201.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200_201.properties
            properties.allow_non_virtual_wan_traffic = AAZBoolType(
                serialized_name="allowNonVirtualWanTraffic",
            )
            properties.auto_scale_configuration = AAZObjectType(
                serialized_name="autoScaleConfiguration",
            )
            properties.express_route_connections = AAZListType(
                serialized_name="expressRouteConnections",
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.virtual_hub = AAZObjectType(
                serialized_name="virtualHub",
                flags={"required": True},
            )

            auto_scale_configuration = cls._schema_on_200_201.properties.auto_scale_configuration
            auto_scale_configuration.bounds = AAZObjectType()

            bounds = cls._schema_on_200_201.properties.auto_scale_configuration.bounds
            bounds.max = AAZIntType()
            bounds.min = AAZIntType()

            express_route_connections = cls._schema_on_200_201.properties.express_route_connections
            express_route_connections.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.express_route_connections.Element
            _element.id = AAZStrType()
            _element.name = AAZStrType(
                flags={"required": True},
            )
            _element.properties = AAZObjectType(
                flags={"client_flatten": True},
            )

            properties = cls._schema_on_200_201.properties.express_route_connections.Element.properties
            properties.authorization_key = AAZStrType(
                serialized_name="authorizationKey",
            )
            properties.enable_internet_security = AAZBoolType(
                serialized_name="enableInternetSecurity",
            )
            properties.enable_private_link_fast_path = AAZBoolType(
                serialized_name="enablePrivateLinkFastPath",
            )
            properties.express_route_circuit_peering = AAZObjectType(
                serialized_name="expressRouteCircuitPeering",
                flags={"required": True},
            )
            properties.express_route_gateway_bypass = AAZBoolType(
                serialized_name="expressRouteGatewayBypass",
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.routing_configuration = AAZObjectType(
                serialized_name="routingConfiguration",
            )
            properties.routing_weight = AAZIntType(
                serialized_name="routingWeight",
            )

            express_route_circuit_peering = cls._schema_on_200_201.properties.express_route_connections.Element.properties.express_route_circuit_peering
            express_route_circuit_peering.id = AAZStrType()

            routing_configuration = cls._schema_on_200_201.properties.express_route_connections.Element.properties.routing_configuration
            routing_configuration.associated_route_table = AAZObjectType(
                serialized_name="associatedRouteTable",
            )
            _CreateHelper._build_schema_sub_resource_read(routing_configuration.associated_route_table)
            routing_configuration.inbound_route_map = AAZObjectType(
                serialized_name="inboundRouteMap",
            )
            _CreateHelper._build_schema_sub_resource_read(routing_configuration.inbound_route_map)
            routing_configuration.outbound_route_map = AAZObjectType(
                serialized_name="outboundRouteMap",
            )
            _CreateHelper._build_schema_sub_resource_read(routing_configuration.outbound_route_map)
            routing_configuration.propagated_route_tables = AAZObjectType(
                serialized_name="propagatedRouteTables",
            )
            routing_configuration.vnet_routes = AAZObjectType(
                serialized_name="vnetRoutes",
            )

            propagated_route_tables = cls._schema_on_200_201.properties.express_route_connections.Element.properties.routing_configuration.propagated_route_tables
            propagated_route_tables.ids = AAZListType()
            propagated_route_tables.labels = AAZListType()

            ids = cls._schema_on_200_201.properties.express_route_connections.Element.properties.routing_configuration.propagated_route_tables.ids
            ids.Element = AAZObjectType()
            _CreateHelper._build_schema_sub_resource_read(ids.Element)

            labels = cls._schema_on_200_201.properties.express_route_connections.Element.properties.routing_configuration.propagated_route_tables.labels
            labels.Element = AAZStrType()

            vnet_routes = cls._schema_on_200_201.properties.express_route_connections.Element.properties.routing_configuration.vnet_routes
            vnet_routes.bgp_connections = AAZListType(
                serialized_name="bgpConnections",
                flags={"read_only": True},
            )
            vnet_routes.static_routes = AAZListType(
                serialized_name="staticRoutes",
            )
            vnet_routes.static_routes_config = AAZObjectType(
                serialized_name="staticRoutesConfig",
            )

            bgp_connections = cls._schema_on_200_201.properties.express_route_connections.Element.properties.routing_configuration.vnet_routes.bgp_connections
            bgp_connections.Element = AAZObjectType()
            _CreateHelper._build_schema_sub_resource_read(bgp_connections.Element)

            static_routes = cls._schema_on_200_201.properties.express_route_connections.Element.properties.routing_configuration.vnet_routes.static_routes
            static_routes.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.express_route_connections.Element.properties.routing_configuration.vnet_routes.static_routes.Element
            _element.address_prefixes = AAZListType(
                serialized_name="addressPrefixes",
            )
            _element.name = AAZStrType()
            _element.next_hop_ip_address = AAZStrType(
                serialized_name="nextHopIpAddress",
            )

            address_prefixes = cls._schema_on_200_201.properties.express_route_connections.Element.properties.routing_configuration.vnet_routes.static_routes.Element.address_prefixes
            address_prefixes.Element = AAZStrType()

            static_routes_config = cls._schema_on_200_201.properties.express_route_connections.Element.properties.routing_configuration.vnet_routes.static_routes_config
            static_routes_config.propagate_static_routes = AAZBoolType(
                serialized_name="propagateStaticRoutes",
                flags={"read_only": True},
            )
            static_routes_config.vnet_local_route_override_criteria = AAZStrType(
                serialized_name="vnetLocalRouteOverrideCriteria",
            )

            virtual_hub = cls._schema_on_200_201.properties.virtual_hub
            virtual_hub.id = AAZStrType()

            tags = cls._schema_on_200_201.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200_201


class _CreateHelper:
    """Helper class for Create"""

    @classmethod
    def _build_schema_sub_resource_create(cls, _builder):
        if _builder is None:
            return
        _builder.set_prop("id", AAZStrType, ".id")

    _schema_sub_resource_read = None

    @classmethod
    def _build_schema_sub_resource_read(cls, _schema):
        if cls._schema_sub_resource_read is not None:
            _schema.id = cls._schema_sub_resource_read.id
            return

        cls._schema_sub_resource_read = _schema_sub_resource_read = AAZObjectType()

        sub_resource_read = _schema_sub_resource_read
        sub_resource_read.id = AAZStrType()

        _schema.id = cls._schema_sub_resource_read.id


__all__ = ["Create"]
